@@ -27,6 +27,7 @@ except ImportError:
 
 log = logging.getLogger(__name__)
 Latest = None  # Enum
+NoVersion = None
 
 
 class State(dict):
@@ -43,10 +44,10 @@ class State(dict):
 
     def __init__(self, ctrl, storage):
         super(State, self).__init__({
-            "projectName": storage.value("startupProject"),
+            "profileName": storage.value("startupProfile"),
             "appRequest": storage.value("startupApplication"),
 
-            # String or callable, returning list of project names
+            # String or callable, returning list of profile names
             "root": None,
 
             # Current error, if any
@@ -55,11 +56,15 @@ class State(dict):
             # Currently commands applications
             "commands": [],
 
-            # Previously loaded project Rez packages
-            "rezProjects": {},
+            # Previously loaded profile Rez packages
+            "rezProfiles": {},
 
             # Currently loaded Rez contexts
             "rezContexts": {},
+
+            # Cache, for performance only
+            "rezEnvirons": {},
+
             "rezApps": odict(),
             "fullCommand": "rez env",
             "serialisationMode": (
@@ -110,7 +115,7 @@ class State(dict):
     def on_enter_booting(self):
         self._ctrl.debug("Booting..")
 
-    def on_enter_selectproject(self):
+    def on_enter_selectprofile(self):
         pass
 
     def on_enter_resolving(self):
@@ -121,8 +126,8 @@ class State(dict):
         util.delay(self.to_ready, 500)
 
     def on_enter_noapps(self):
-        project = self["projectName"]
-        self._ctrl.debug("No applications were found for %s" % project)
+        profile = self["profileName"]
+        self._ctrl.debug("No applications were found for %s" % profile)
 
     def on_enter_loading(self):
         self._ctrl.debug("Loading..")
@@ -149,6 +154,23 @@ class _State(transitions.State):
         return not self.__eq__(other)
 
 
+class _Stream(object):
+    def __init__(self, ctrl, stream, level):
+        self._ctrl = ctrl
+        self._stream = stream
+        self._level = level
+
+    def write(self, text):
+        self._stream.write(text) if self._stream else None
+        self._ctrl.logged.emit(text, self._level)
+
+    def fileno(self):
+        return 0
+
+    def close(self):
+        return
+
+
 class Controller(QtCore.QObject):
     state_changed = QtCore.Signal(_State)
     logged = QtCore.Signal(str, int)  # message, level
@@ -157,8 +179,8 @@ class Controller(QtCore.QObject):
     # One or more packages have changed on disk
     repository_changed = QtCore.Signal()
 
-    project_changed = QtCore.Signal(
-        str, object, bool)  # project, version, refreshed
+    profile_changed = QtCore.Signal(
+        str, object, bool)  # profile, version, refreshed
 
     application_changed = QtCore.Signal()
 
@@ -174,20 +196,20 @@ class Controller(QtCore.QObject):
         _State("errored", help="Something has gone wrong"),
         _State("launching", help="An application is launching"),
         _State("ready", help="Awaiting user input"),
-        _State("noproject", help="A given project package was not found"),
+        _State("noprofiles", help="Allzpark did not find any profiles at all"),
         _State("noapps", help="There were no applications to choose from"),
         _State("notresolved", help="Rez couldn't resolve a request"),
         _State("pkgnotfound", help="One or more packages was not found"),
     ]
 
-    def __init__(self, storage, parent=None):
+    def __init__(self, storage, stdio=None, stderr=None, parent=None):
         super(Controller, self).__init__(parent)
 
         state = State(self, storage)
 
         models = {
-            "projectVersions": QtCore.QStringListModel(),
-            "projectNames": QtCore.QStringListModel(),
+            "profileVersions": QtCore.QStringListModel(),
+            "profileNames": QtCore.QStringListModel(),
             "apps": model.ApplicationModel(),
 
             # Docks
@@ -199,15 +221,11 @@ class Controller(QtCore.QObject):
 
         timers = {
             "commandsPoller": QtCore.QTimer(self),
-            "applicationSelector": QtCore.QTimer(self),
         }
 
         timers["commandsPoller"].timeout.connect(self.on_tasks_polled)
         timers["commandsPoller"].start(500)
 
-        timers["applicationSelector"].setSingleShot(True)
-        timers["applicationSelector"].timeout.connect(self._select_application)
-
         # Initialize the state machine
         self._machine = transitions.Machine(
             model=state,
@@ -247,8 +265,8 @@ class Controller(QtCore.QObject):
         return self._state["error"]
 
     @property
-    def current_project(self):
-        return self._state["projectName"]
+    def current_profile(self):
+        return self._state["profileName"]
 
     @property
     def current_application(self):
@@ -262,12 +280,39 @@ class Controller(QtCore.QObject):
         return self._state["rezContexts"][app_request].to_dict()
 
     def environ(self, app_request):
+        """Fetch the environment of a context
+
+        NOTE: These can get very expensive. They call on every
+              package.py:commands() in a resolved context, which can
+              be in the tens to hundreds. Add to that the fact that
+              these functions can perform any arbitrary task, including
+              writing to disk or performing expensive calculations,
+              such as resolving their own contexts for various reasons.
+
+        TODO: This should be async, the GUI should help the user
+              understand that the environment is loading and is
+              going to be ready soon. They should also only incur
+              cost when the user is actually looking at the
+              environment tab.
+
+        """
+
+        env = self._state["rezEnvirons"]
+        ctx = self._state["rezContexts"]
+
         try:
-            return self._state["rezContexts"][app_request].get_environ()
-        except rez.ResolvedContextError:
-            return {
-                "error": "Failed context"
-            }
+            return env[app_request]
+
+        except KeyError:
+            try:
+                environ = ctx[app_request].get_environ()
+                env[app_request] = environ
+                return environ
+
+            except rez.ResolvedContextError:
+                return {
+                    "error": "Failed context"
+                }
 
     def resolved_packages(self, app_request):
         return self._state["rezContexts"][app_request].resolved_packages
@@ -306,8 +351,11 @@ class Controller(QtCore.QObject):
         """
 
         # Potentially overridden by the below
-        self._state["error"] = "".join(traceback.format_tb(tb))
+        self._state["error"] = "".join(
+            traceback.format_tb(tb) + [str(value)]
+        )
         self._state.to_errored()
+        self.error(self._state["error"])
 
         if rez.PackageNotFoundError is type:
             package = value.value.rsplit(": ", 1)[-1]
@@ -315,7 +363,7 @@ class Controller(QtCore.QObject):
             message = """
                 <h2><font color=\"red\">:(</font></h2>
 
-                Package '{package}' is required by this project,
+                Package '{package}' is required by this profile,
                 but could <font color=\"red\">not be found.</font>
                 <br>
                 <br>
@@ -342,7 +390,7 @@ class Controller(QtCore.QObject):
             message = """
                 <h2><font color=\"red\">:(</font></h2>
 
-                Package '{package}' is required by this project,
+                Package '{package}' is required by this profile,
                 but could <font color=\"red\">not be found.</font>
                 <br>
                 <br>
@@ -388,6 +436,9 @@ class Controller(QtCore.QObject):
     # Methods
     # ----------------
 
+    def stdio(self, stream, level=logging.INFO):
+        return _Stream(self, stream, level)
+
     def find(self, family, range_=None):
         """Find packages, relative Allzpark state
 
@@ -481,6 +532,7 @@ class Controller(QtCore.QObject):
         command += ["--", tool]
 
         self._state["fullCommand"] = " ".join(command)
+
         self.command_changed.emit(self._state["fullCommand"])
 
     def _package_filter(self):
@@ -496,12 +548,12 @@ class Controller(QtCore.QObject):
     def reset(self, root=None, on_success=lambda: None):
         """Initialise controller with `root`
 
-        Projects are listed at `root` and matched
+        Profiles are listed at `root` and matched
         with its corresponding Rez package.
 
         Arguments:
-            root (str): Absolute path to projects on disk, or callable
-                returning names of projects
+            root (str): Absolute path to profiles on disk, or callable
+                returning names of profiles
 
         """
 
@@ -510,68 +562,71 @@ class Controller(QtCore.QObject):
         assert root, "Tried resetting without a root, this is a bug"
 
         def do():
-            projects = dict()
-            default_project = None
+            profiles = dict()
+            default_profile = None
 
-            for name in self.list_projects(root):
+            for name in self.list_profiles(root):
 
-                # Find project package
+                # Find profile package
                 package = None
                 for package in self.find(name):
 
-                    if name not in projects:
-                        projects[name] = dict()
+                    if name not in profiles:
+                        profiles[name] = dict()
 
-                    projects[name][str(package.version)] = package
-                    projects[name][Latest] = package
+                    profiles[name][str(package.version)] = package
+                    profiles[name][Latest] = package
 
                 if package is None:
                     package = model.BrokenPackage(name)
-                    projects[name] = {
+                    profiles[name] = {
                         "0.0": package,
                         Latest: package,
                     }
 
                 # Default to latest of last
-                default_project = name
+                default_profile = name
 
-            self._state["rezProjects"].update(projects)
-            self._models["projectNames"].setStringList(list(projects))
-            self._models["projectNames"].layoutChanged.emit()
+            self._state["rezProfiles"].update(profiles)
+            self._models["profileNames"].setStringList(list(profiles))
+            self._models["profileNames"].layoutChanged.emit()
 
             # On resetting after startup, there will be a
-            # currently selected project that may differ from
-            # the startup project.
-            current_project = self._state["projectName"]
+            # currently selected profile that may differ from
+            # the startup profile.
+            current_profile = self._state["profileName"]
 
-            if current_project and current_project not in projects:
-                self.warning("Startup project '%s' did not exist"
-                             % current_project)
-                current_project = None
+            if current_profile and current_profile not in profiles:
+                self.warning("Startup profile '%s' did not exist"
+                             % current_profile)
+                current_profile = None
 
             # The user has never opened the GUI before,
             # or user preferences has been wiped.
-            if not current_project:
-                current_project = default_project
+            if not current_profile:
+                current_profile = default_profile
 
-            # Fallback
-            if not current_project:
-                current_project = "no_project"
-
-            self._state["projectName"] = current_project
+            self._state["profileName"] = current_profile
             self._state["root"] = root
 
             self._state.to_ready()
             self.resetted.emit()
 
         def _on_success():
-            self.select_project(self._state["projectName"])
+            profile = not self._state["profileName"]
+
+            if profile:
+                self._state.to_noprofiles()
+            else:
+                self.select_profile(profile)
+
             on_success()
 
         def _on_failure(error, trace):
             raise error
 
         self._state["rezContexts"].clear()
+        self._state["rezEnvirons"].clear()
         self._state["rezApps"].clear()
 
         # Rez stores file listings and more
@@ -643,7 +698,6 @@ class Controller(QtCore.QObject):
             def on_error(error):
                 # Forward error from Command()
                 raise error
-                # raise rez.ResolvedContextError(str(error))
 
             cmd = Command(
                 context=rez_context,
@@ -728,53 +782,53 @@ class Controller(QtCore.QObject):
         return None
 
     def debug(self, message):
-        log.debug(message)
         self.logged.emit(message, logging.DEBUG)
 
     def info(self, message):
-        log.info(message)
         self.logged.emit(message, logging.INFO)
 
     def warning(self, message):
-        log.warning(message)
         self.logged.emit(message, logging.WARNING)
 
     def error(self, message):
-        log.error(message)
         self.logged.emit(str(message), logging.ERROR)
 
-    def list_projects(self, root=None):
+    def list_profiles(self, root=None):
         root = root or self._state["root"]
         assert root, "Tried listing without a root, this is a bug"
 
         if isinstance(root, (tuple, list)):
-            projects = root
+            profiles = root
 
         elif callable(root):
             try:
-                projects = root()
+                profiles = root()
 
             except Exception:
-                if log.level < logging.INFO:
+                if log.level == logging.DEBUG:
                     traceback.print_exc()
 
-                self.error("Could not find projects in %s" % root)
-                projects = []
+                self.error("Could not find profiles in %s" % root)
+                profiles = []
 
         # Facilitate accidental empty family names, e.g. None or ''
-        projects = list(filter(None, projects))
+        profiles = list(filter(None, profiles))
 
-        return projects
+        return profiles
 
     @util.async_
-    def select_project(self, project_name, version_name=Latest):
+    def select_profile(self, profile_name, version_name=Latest):
 
         # Wipe existing data
         self._models["apps"].reset()
         self._models["context"].reset()
         self._models["environment"].reset()
         self._models["packages"].reset()
-        self._models["projectVersions"].setStringList([])
+        self._models["profileVersions"].setStringList([])
+
+        self._state["rezContexts"].clear()
+        self._state["rezEnvirons"].clear()
+        self._state["rezApps"].clear()
 
         def on_apps_found(apps):
             if not apps:
@@ -782,11 +836,11 @@ class Controller(QtCore.QObject):
                 <h2><font color=\"red\">:(</font></h2>
                 <br>
                 <br>
-                The project was found, but no applications.
+                The profile was found, but no applications.
                 <br>
                 <br>
-                The project didn't specify an application for you to use.<br>
-                This is likely due to a misconfigured project. Don't forget<br>
+                The profile didn't specify an application for you to use.<br>
+                This is likely due to a misconfigured profile. Don't forget<br>
                 to provide one or more packages as <i>weak references</i>.
                 <br>
                 <br>
@@ -805,47 +859,56 @@ class Controller(QtCore.QObject):
             raise error
 
         try:
-            project_versions = self._state["rezProjects"][project_name]
-            active_project = project_versions[version_name]
-
-            refreshed = self._state["projectName"] == project_name
-            self._state["projectName"] = project_name
-            self.project_changed.emit(
-                project_name,
-                version_name,
-                refreshed
-            )
-
-            if isinstance(active_project, model.BrokenPackage):
-                raise rez.PackageNotFoundError(
-                    "package not found: %s" % project_name
-                )
-
-            # Update versions model
-            versions = list(filter(None, project_versions))  # Exclude "Latest"
-            self._models["projectVersions"].setStringList(versions)
-
-            self._state.to_loading()
-            util.defer(
-                self._list_apps,
-                args=[active_project],
-                on_success=on_apps_found,
-                on_failure=on_apps_not_found,
-            )
+            profile_versions = self._state["rezProfiles"][profile_name]
+            active_profile = profile_versions[version_name]
 
         except KeyError:
-            self._state.to_notresolved()
+            # This can only happen if somehow the view decided to pass
+            # along the name and version of a profile that didn't exist.
+            profile_name = self._state["profileName"]
+            profile_versions = self._state["rezProfiles"][profile_name]
+            active_profile = profile_versions[version_name]
+
+            if profile_name:
+                self.warning("%s was not found" % profile_name)
+            else:
+                self.error("select_profile was passed an empty string")
+
+        refreshed = self._state["profileName"] == profile_name
+
+        # TODO: This isn't clear.
+        # We can't pass a native Rez Version object, but we also can't
+        # simply str() that and BrokenPackage.version, as those would
+        # be None, which is the equivalent of a NoVersion object.
+        version_name = active_profile.version
+        version_name = str(version_name) if version_name else NoVersion
+
+        self._state["profileName"] = profile_name
+        self.profile_changed.emit(
+            profile_name,
+            version_name,
+            refreshed
+        )
+
+        if isinstance(active_profile, model.BrokenPackage):
+            raise rez.PackageNotFoundError(
+                "package not found: %s" % profile_name
+            )
+
+        # Update versions model
+        versions = list(filter(None, profile_versions))  # Exclude "Latest"
+        versions.reverse()  # Latest first
+        self._models["profileVersions"].setStringList(versions)
+
+        self._state.to_loading()
+        util.defer(
+            self._list_apps,
+            args=[active_profile],
+            on_success=on_apps_found,
+            on_failure=on_apps_not_found,
+        )
 
     def select_application(self, app_request):
-        self.__app_request = app_request
-
-        # Delay acting on application select, to avoid
-        # processing to happen too frequently and affect
-        # the user experience.
-        self._timers["applicationSelector"].start(250)
-
-    def _select_application(self):
-        app_request = self.__app_request
         self._state["appRequest"] = app_request
 
         try:
@@ -866,7 +929,7 @@ class Controller(QtCore.QObject):
         tools = self._models["apps"].find(app_request)["tools"]
         self._state["tool"] = tools[0]
 
-        # Use this application on next launch or change of project
+        # Use this application on next launch or change of profile
         self.update_command()
         self._state.store("startupApplication", app_request)
         self.application_changed.emit()
@@ -896,8 +959,8 @@ class Controller(QtCore.QObject):
 
         return paths
 
-    def _list_apps(self, project):
-        # Each app has a unique context relative the current project
+    def _list_apps(self, profile):
+        # Each app has a unique context relative the current profile
         # Find it, and keep track of it.
 
         apps = []
@@ -926,30 +989,32 @@ class Controller(QtCore.QObject):
                                  "missing `allzparkconfig.applications`")
 
         if not apps:
-            apps[:] = allzparkconfig.applications_from_package(project)
+            apps[:] = allzparkconfig.applications_from_package(profile)
 
         # Optional patch
         patch = self._state.retrieve("patch", "").split()
-
-        # Exclude filtered
         package_filter = self._package_filter()
-        for app in apps[:]:
-            if package_filter.excludes(app):
-                apps.remove(app)
 
         contexts = odict()
         with util.timing() as t:
-            for app_package in apps:
-                variants = list(project.iter_variants())
+            for app_request in apps:
+                app_request = rez.PackageRequest(app_request)
+                app_package = rez.find_latest(app_request.name,
+                                              range_=app_request.range)
+
+                if package_filter.excludes(app_package):
+                    continue
+
+                variants = list(profile.iter_variants())
                 variant = variants[0]
 
                 if len(variants) > 1:
                     # Unsure of whether this is desirable. It would enable
-                    # a project per platform, or potentially other kinds
+                    # a profile per platform, or potentially other kinds
                     # of special-purpose situations. If you see this,
                     # and want this, submit an issue with your use case!
                     self.warning(
-                        "Projects with multiple variants are unsupported. "
+                        "Profiles with multiple variants are unsupported. "
                         "Using first found: %s" % variant
                     )
 
@@ -999,7 +1064,9 @@ class Controller(QtCore.QObject):
                 if rez_context.success:
                     self.warning(
                         "This shouldn't have happened, "
-                        "I was expecting a broken context here."
+                        "I was expecting a broken context here. "
+                        "Please report this to "
+                        "https://github.com/mottosso/allzpark/issues/66"
                     )
 
                 self.error(
@@ -1012,38 +1079,25 @@ class Controller(QtCore.QObject):
 
         self.debug("Resolved all contexts in %.2f seconds" % t.duration)
 
-        # Find resolved app version
-        # E.g. maya -> maya-2018.0.1
-        app_packages = []
+        # Hide hidden
+        visible_apps = []
         show_hidden = self._state.retrieve("showHiddenApps")
-        for app_request, context in contexts.items():
-            for package in context.resolved_packages or []:
-                if package.name in [a.name for a in apps]:
-                    break
-            else:
-                package = model.BrokenPackage(app_request)
-
+        for request, package in self._state["rezApps"].items():
             data = allzparkconfig.metadata_from_package(package)
             hidden = data.get("hidden", False)
 
             if hidden and not show_hidden:
-                package.hidden = True
                 continue
 
-            if isinstance(context, model.BrokenContext):
-                package.broken = True
-
-            app_packages += [package]
+            visible_apps += [package]
 
         self._state["rezContexts"] = contexts
-        return app_packages
+        return visible_apps
 
     def graph(self):
         context = self._state["rezContexts"][self._state["appRequest"]]
         graph_str = context.graph(as_dot=True)
 
-        self.debug("Generating graph from %s" % graph_str)
-
         tempdir = tempfile.mkdtemp()
         fname = os.path.join(tempdir, "graph.png")
 
@@ -1114,12 +1168,22 @@ class Command(QtCore.QObject):
         self.thread.start()
 
     def _execute(self):
+        startupinfo = None
+        no_console = hasattr(allzparkconfig, "__noconsole__")
+
+        # Windows-only
+        # Prevent additional windows from appearing when running
+        # Allzpark without a console, e.g. via pythonw.exe.
+        if no_console and hasattr(subprocess, "STARTUPINFO"):
+            startupinfo = subprocess.STARTUPINFO()
+            startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
+
         kwargs = {
             "command": self.cmd,
             "stdout": subprocess.PIPE,
             "stderr": subprocess.PIPE,
-            "detached": self.detached,
             "parent_environ": None,
+            "startupinfo": startupinfo
         }
 
         context = self.context
