@@ -67,6 +67,8 @@ class AbstractDockWidget(QtWidgets.QDockWidget):
 
 
 class App(AbstractDockWidget):
+    """Aggregated information about the currently selected profile"""
+
     icon = "Alert_Info_32"
 
     def __init__(self, ctrl, parent=None):
@@ -94,12 +96,6 @@ class App(AbstractDockWidget):
                 qargparse.Choice("tool", help=(
                     "Which executable within the context of this application"
                 )),
-                qargparse.Boolean("detached", help=(
-                    "Spawn a dedicated console for this executable\n"
-                    "Typically only necessary for console applications. "
-                    "If you find that an executable doesn't provide a window, "
-                    "such as mayapy, then you probably want detached."
-                )),
             ]),
 
             # Shortcuts
@@ -169,7 +165,7 @@ class App(AbstractDockWidget):
         self._ctrl.launch()
 
     def on_arg_changed(self, arg):
-        if arg["name"] not in ("detached", "tool"):
+        if arg["name"] not in ("tool",):
             return
 
         ctrl = self._ctrl
@@ -736,20 +732,8 @@ class Commands(AbstractDockWidget):
         model = index.model()
 
         menu = QtWidgets.QMenu(self)
-        kill = QtWidgets.QAction("Kill", menu)
-        copy_command = QtWidgets.QAction("Copy command", menu)
         copy_pid = QtWidgets.QAction("Copy pid", menu)
 
-        def on_kill():
-            name = model.data(index, "cmd")
-            if not model.data(index, "running"):
-                self.message.emit("%s isn't running" % name)
-                return
-
-            self.message.emit("Killing %s" % name)
-            command = model.data(index, "object")
-            command.kill()
-
         def on_copy_pid():
             clipboard = QtWidgets.QApplication.instance().clipboard()
             command = model.data(index, "object")
@@ -757,30 +741,15 @@ class Commands(AbstractDockWidget):
             clipboard.setText(pid)
             self.message.emit("Copying %s" % pid)
 
-        def on_copy_command():
-            clipboard = QtWidgets.QApplication.instance().clipboard()
-            cmd = model.data(index, "niceCmd")
-            clipboard.setText(cmd)
-            self.message.emit("Copying %s" % cmd)
-
         if model.data(index, "running") != "killed":
-            kill.triggered.connect(on_kill)
-            copy_command.triggered.connect(on_copy_command)
             copy_pid.triggered.connect(on_copy_pid)
         else:
-            kill.setEnabled(False)
-            copy_command.setEnabled(False)
             copy_pid.setEnabled(False)
 
-        # TODO: This doesn't work on Windows, and possibly not
-        # even Linux, due to what's really being managed here
-        # is the *parent* of a given process, like Maya, as
-        # opposed to the actual Maya process.
-        menu.addAction(kill) if os.name != "nt" else None
+        # See https://github.com/mottosso/allzpark/issues/88
+        # menu.addAction(copy_pid)
 
-        menu.addAction(copy_pid)
         menu.move(QtGui.QCursor.pos())
-
         menu.show()
 
 
@@ -801,6 +770,7 @@ class EnvironmentEditor(QtWidgets.QWidget):
         font.setPointSize(10)
 
         widgets["textEdit"].setFont(font)
+        widgets["textEdit"].setLineWrapMode(QtWidgets.QTextEdit.NoWrap)
 
         layout = QtWidgets.QVBoxLayout(self)
         layout.setContentsMargins(0, 0, 0, 0)
@@ -823,6 +793,9 @@ class EnvironmentEditor(QtWidgets.QWidget):
         self.on_focus_lost()
 
     def from_environment(self, environ):
+        if not environ:
+            return
+
         text = "\n".join([
             "%s=%s" % (key, value)
             for key, value in environ.items()
@@ -891,6 +864,7 @@ class CssEditor(QtWidgets.QWidget):
         font.setPointSize(10)
 
         widgets["textEdit"].setFont(font)
+        widgets["textEdit"].setLineWrapMode(QtWidgets.QTextEdit.NoWrap)
 
         layout = QtWidgets.QVBoxLayout(self)
         layout.setContentsMargins(0, 0, 0, 0)
@@ -924,8 +898,8 @@ class Preferences(AbstractDockWidget):
     icon = "Action_GoHome_32"
 
     options = [
-        qargparse.Info("startupProject", help=(
-            "Load this project on startup"
+        qargparse.Info("startupProfile", help=(
+            "Load this profile on startup"
         )),
         qargparse.Info("startupApplication", help=(
             "Load this application on startup"
@@ -957,7 +931,7 @@ class Preferences(AbstractDockWidget):
         )),
         qargparse.Boolean("showAllApps", help=(
             "List everything from allzparkconfig:applications\n"
-            "not just the ones specified for a given project."
+            "not just the ones specified for a given profile."
         )),
         qargparse.Boolean("showHiddenApps", help=(
             "Show apps with metadata['hidden'] = True"
@@ -1130,16 +1104,118 @@ class SlimTableView(QtWidgets.QTableView):
                 self.customContextMenuRequested.emit(event.pos())
 
 
+class PushButtonWithMenu(QtWidgets.QPushButton):
+    def __init__(self, *args, **kwargs):
+        super(PushButtonWithMenu, self).__init__(*args, **kwargs)
+        self.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
+
+    def mousePressEvent(self, event):
+        """Do request a context menu, but on mouse *press*, not release"""
+
+        # Important, call this first to have the item be selected,
+        # as per default, and *then* ask for a context menu. That
+        # way, the menu and selection aligns.
+        try:
+            return super(PushButtonWithMenu, self).mousePressEvent(event)
+
+        finally:
+            if event.button() == QtCore.Qt.RightButton:
+                self.customContextMenuRequested.emit(event.pos())
+
+
+class LineEditWithCompleter(QtWidgets.QLineEdit):
+    changed = QtCore.Signal(str)
+
+    def __init__(self, parent=None):
+        super(LineEditWithCompleter, self).__init__(parent)
+
+        proxy = QtCore.QSortFilterProxyModel(self)
+        proxy.setFilterCaseSensitivity(QtCore.Qt.CaseInsensitive)
+
+        completer = QtWidgets.QCompleter(proxy, self)
+        completer.setCaseSensitivity(QtCore.Qt.CaseInsensitive)
+        completer.setCompletionMode(completer.UnfilteredPopupCompletion)
+        completer.setMaxVisibleItems(15)
+
+        self.setCompleter(completer)
+        self.editingFinished.connect(self.onEditingFinished)
+
+        self._completer = completer
+        self._focused = False
+        self._proxy = proxy
+        self._current = None
+
+    def setModel(self, model):
+        self._proxy.setSourceModel(model)
+        self._completer.setModel(self._proxy)
+
+    def setText(self, text):
+
+        # Keep track of a "default" such that we can revert back to
+        # it following a bad completion.
+        self._current = text
+
+        return super(LineEditWithCompleter, self).setText(text)
+
+    def resetText(self):
+        self.setText(self._current)
+
+    def mousePressEvent(self, event):
+        super(LineEditWithCompleter, self).mousePressEvent(event)
+
+        # Automatically show dropdown on select
+        self._completer.complete()
+        self.selectAll()
+
+    def onEditingFinished(self):
+        # For whatever reason, the completion prefix isn't updated
+        # when coming from the user selecting an item from the
+        # completion listview. (Windows 10, PySide2, Python 3.7)
+        self._completer.setCompletionPrefix(self.text())
+
+        suggested = self._completer.currentCompletion()
+
+        if not suggested:
+            return self.resetText()
+
+        self.changed.emit(suggested)
+
+
 class MenuWithTooltip(QtWidgets.QMenu):
     """QMenu typically doesn't draw tooltips"""
 
     def event(self, event):
         if event.type() == QtCore.QEvent.ToolTip and self.activeAction() != 0:
-            QtWidgets.QToolTip.showText(
-                event.globalPos(),
-                self.activeAction().toolTip()
-            )
-        else:
+
+            try:
+                tooltip = self.activeAction().toolTip()
+            except AttributeError:
+                # Can sometimes return None
+                pass
+            else:
+                QtWidgets.QToolTip.showText(
+                    event.globalPos(),
+                    tooltip
+                )
+
+        # Account for QStatusBar queries about tooltip
+        # These typically are only available on hovering
+        # an action for some time, which isn't reflected
+        # in querying the menu itself for a tooltip.
+        elif event.type() == QtCore.QEvent.MouseMove:
+            try:
+                tooltip = self.activeAction().toolTip()
+
+                # Some tooltips are multi-line, and the statusbar
+                # typically ignores newlines and writes it all out
+                # as one long line.
+                tooltip = tooltip.splitlines()[0]
+
+                self.setToolTip(tooltip)
+
+            except (AttributeError, IndexError):
+                pass
+
             QtWidgets.QToolTip.hideText()
 
         return super(MenuWithTooltip, self).event(event)
